#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "mdb.h"

static int pcheck(void *);
static FILE *fp;
#define MAGIC ((size_t) 0xDEADBEEF)

static void *qmmalloc(char *, int, size_t);
static void *qmcalloc(char *, int, size_t, size_t);
static void *qmrealloc(char *, int, void *, size_t);
static void qmfree(char *, int, void *);
static void *mmmalloc(char *, int, size_t);
static void *mmcalloc(char *, int, size_t, size_t);
static void *mmrealloc(char *, int, void *, size_t);
static void mmfree(char *, int, void *);

void * (*mmalloc)(char *, int, size_t) = mmmalloc;
void * (*mcalloc)(char *, int, size_t, size_t) = mmcalloc;
void * (*mrealloc)(char *, int, void *, size_t) = mmrealloc;
void (*mfree)(char *, int, void *) = mmfree;

static void
init_fp(void) {
	if (!fp) {
		fp = fopen("mdb.out", "w");
		if (!fp)
			fp = stderr;
	}
}

static void *
mmcalloc(char *file, int lno, size_t nitems, size_t nbytes) {
	char *ret;
	size_t rlen;
	size_t len = nbytes * nitems;

	init_fp();

	fprintf(fp, "m %d %s %d ",
		(int) len, file, lno);
	fflush(fp);

	if (len % sizeof(size_t)) {
		len += sizeof(size_t) - (len % sizeof(size_t));
	}

	rlen = len + (3 * sizeof(size_t));
	ret = malloc(rlen);
	if (ret) {
		*((size_t *) ret) = len;
		*(1 + (size_t *) ret) = MAGIC;
		*(2 + (size_t *) (ret + len)) = MAGIC;
		memset(ret + 2 * sizeof(size_t), 0x00, len);
		fprintf(fp, "%p\n", ret + 2 * sizeof(size_t));
		fflush(fp);
		return ret + 2 * sizeof(size_t);
	} else {
		fprintf(fp, "null\n");
		fflush(fp);
		return ret;
	}
}

static void *
mmmalloc(char *file, int lno, size_t len) {
	char *ret;
	size_t rlen;

	init_fp();

	fprintf(fp, "m %d %s %d ",
		(int) len, file, lno);
	fflush(fp);

	if (len % sizeof(size_t)) {
		len += sizeof(size_t) - (len % sizeof(size_t));
	}

	rlen = len + (3 * sizeof(size_t));
	ret = malloc(rlen);
	if (ret) {
		*((size_t *) ret) = len;
		*(1 + (size_t *) ret) = MAGIC;
		*(2 + (size_t *) (ret + len)) = MAGIC;
		memset(ret + 2 * sizeof(size_t), 0xFF, len);
		fprintf(fp, "%p\n", ret + 2 * sizeof(size_t));
		fflush(fp);
		return ret + 2 * sizeof(size_t);
	} else {
		fprintf(fp, "null\n");
		fflush(fp);
		return ret;
	}
}

static void *
mmrealloc(char *file, int lno, void *oldptr, size_t len) {
	char *ret;
	char *ptr;
	size_t oldlen = 0;

	init_fp();

	fprintf(fp, "r %d %s %d %p ",
		(int) len, file, lno, oldptr);
	fflush(fp);
	if (len % sizeof(size_t)) {
		len += sizeof(size_t) - (len % sizeof(size_t));
	}
	pcheck(oldptr);

	ptr = ((char *) oldptr) - 2 * sizeof(size_t);

	if (oldptr == 0) {
		ret = malloc(len + 3 * sizeof(size_t));
	} else if (oldptr && !len) {
		ret = 0;
		free(ptr);
	} else {
		oldlen = *((size_t *) ptr);
		ret = realloc(ptr, len + 3 * sizeof(size_t));
	}
	if (ret) {
		fprintf(fp, "%p\n", ret + 2 * sizeof(size_t));
		*((size_t *)ret) = len;
		*(1 + (size_t *)ret) = MAGIC;
		*(2 + (size_t *) (ret + len)) = MAGIC;
		memset(ret + 2 * sizeof(size_t) + oldlen, 0xFF, len - oldlen);
		fflush(fp);
		return ret + 2 * sizeof(size_t);
	} else {
		fprintf(fp, "null\n");
		fflush(fp);
		return ret;
	}
}

static void
mmfree(char *file, int lno, void *oldptr) {
	char *ptr;
	init_fp();

	if (oldptr == 0)
		return;
	fprintf(fp, "f %p %s %d\n",
		oldptr, file, lno);
	ptr = oldptr;
	pcheck(oldptr);

	ptr -= 2 * sizeof(size_t);
	memset(ptr, 0xFF, *((size_t *)ptr) + 3 * sizeof(size_t));
	free(ptr);
	fflush(fp);
}

static int
pcheck(void *ptr) {
	size_t len;
	size_t *sp;
	int ret = 0;

	sp = ptr;

	if (!sp)
		return ret;
	--sp;
	if (*sp != MAGIC) {
		fprintf(fp, "u %p\n", ptr);
		ret = 1;
	}
	--sp;
	len = *sp;
	sp += (len / sizeof(size_t)) + 2;
	if (*sp != MAGIC) {
		fprintf(fp, "o %p\n", ptr);
		ret = 1;
	}
	return ret;
}

static void
qmfree(char *file, int lno, void *oldptr) {
	if (oldptr)
		free(oldptr);
}

static void *
qmrealloc(char *file, int lno, void *oldptr, size_t len) {
	if (oldptr && !len) {
		free(oldptr);
		return 0;
	}
	if (oldptr && len)
		return realloc(oldptr, len);

	if (!oldptr && len)
		return malloc(len);

	return 0;
}

static void *
qmmalloc(char *file, int lno, size_t len) {
	return malloc(len);
}

static void *
qmcalloc(char *file, int lno, size_t s1, size_t s2) {
	return calloc(s1, s2);
}

extern void
mdb_on(void) {
	mmalloc = mmmalloc;
	mrealloc = mmrealloc;
	mcalloc = mmcalloc;
	mfree = mmfree;
}

extern void
mdb_off(void) {
	mmalloc = qmmalloc;
	mrealloc = qmrealloc;
	mcalloc = qmcalloc;
	mfree = qmfree;
}
