#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#define MDB
#include "mdb.h"

#include "sz.h"
#define WEB_PRIVATE
#include "web.h"
#include "webattr.h"

struct webattr {
	unsigned char magic[2];
	int pairs;
	int size;
	sz **names;
	sz **values;
};

void
webattrfree(webattr *wa) {
	int i;

	if (!wa)
		return;

	for (i = 0; i < wa->size; ++i) {
		szfree(wa->names[i]);
		szfree(wa->values[i]);
	}
	free(wa->names);
	free(wa->values);
	free(wa);
}

webattr *
webattrnew(void) {
	webattr *wa = malloc(sizeof(webattr));
	if (!wa)
		return 0;

	wa->magic[0] = (unsigned char) -1;
	wa->magic[1] = WEBATTR_MAGIC;
	wa->pairs = 0;
	wa->size = 0;
	wa->names = 0;
	wa->values = 0;

	return wa;
}

webattr *
webattrmake(void *v, ...) {
	sz *s1 = szgetp(v);
	webattr *wa = 0;
	int i;

	if (!s1)
		return 0;

	wa = webattrnew();
	if ((i = szindex(s1, '=')) >= 0) {
		sz *tmp = sztail(s1, i + 1);
		sztrunc(s1, i);
		webattraddnv(wa, s1, tmp);
	} else {
		sz *s2;
		va_list ap;
		va_start(ap, v);
		s2 = szgetp(va_arg(ap, void *));
		va_end(ap);
		webattraddnv(wa, s1, s2);
		szkill(s2);
	}
	szkill(s1);

	/* webattrfdebug(stderr, wa); */
	return wa;
}

void
webattraddnv(webattr *a, sz *name, sz *value) {
	sz **tmp;
	if (!a)
		return;

	if (a->pairs >= a->size) {
		tmp = realloc(a->names, (a->pairs + 1) * sizeof(sz *));
		if (tmp) {
			a->names = tmp;
		} else {
			return;
		}

		tmp = realloc(a->values, (a->pairs + 1) * sizeof(sz *));
		if (tmp) {
			a->values = tmp;
		} else {
			return;
		}

		a->names[a->pairs] = 0;
		a->values[a->pairs] = 0;
		++a->size;
	}
	a->names[a->pairs] = szdup(name);
	a->values[a->pairs] = szdup(value);
	++a->pairs;
}

void
webattrcat(webattr *a, webattr *b) {
	int i;
	if (a->pairs + b->pairs > a->size) {
		sz **tmp;
		tmp = realloc(a->names, (a->pairs + b->pairs) * sizeof(sz *));
		if (tmp) {
			a->names = tmp;
		} else {
			return;
		}

		tmp = realloc(a->values, (a->pairs + b->pairs) * sizeof(sz *));
		if (tmp) {
			a->values = tmp;
		} else {
			return;
		}

		for (i = 0; i < b->pairs; ++i) {
			a->names[a->pairs + i] = 0;
			a->values[a->pairs + i] = 0;
		}
		a->size = a->pairs + b->pairs;
	}
	for (i = 0; i < b->pairs; ++i) {
		webattraddnv(a, b->names[i], b->values[i]);
	}
}

void
webattrfdebug(FILE *f, webattr *a) {
	int i;
	fprintf(f, "%p: webattr [%d/%d]\n", (void *) a, a->pairs, a->size);
	for (i = 0; i < a->pairs; ++i) {
		fprintf(f, "\t'%s' => '%s'\n",
			a->names[i] ? szdata(a->names[i]) : "null",
			a->values[i] ? szdata(a->values[i]) : "null");
	}
}

void
webattrfprint(FILE *f, webattr *a) {
	int i;
	if (!a)
		return;
	for (i = 0; i < a->pairs; ++i) {
		fprintf(f, " %s=\"%s\"",
			a->names[i] ? szdata(a->names[i]) : "null",
			a->values[i] ? szdata(a->values[i]) : "null");
	}
}

webattr *
webattrdup(webattr *wa) {
	webattr *tmp = webattrnew();
	webattrcat(tmp, wa);
	return tmp;
}

webattr *
webattrgetp(void *v) {
	unsigned char *u = v;
	if ((u[0] == (unsigned char) -1) && u[1] == WEBATTR_MAGIC) {
		return v;
	}
	return 0;
}
